#ifndef spiRam_h
#define spiRam_h

// ceptimus 2015-12-05
// fast, small, and simple interrupt-driven routines to use the 128K SPI RAM chip 23LC1024 on a Uno, Nano, Micro or other AtMega328 chip
// code and connections will need tweaking for use with other Arduino family members
// connections chip pin number - Arduino reference:
// 1 - D10, 2 - D12, 3 - 5V, 4 - Gnd, 5 - D11, 6 - D13, 7 - 5V, 8 - 5V
// if you want to be correct you can put 100K or so pull-up-resistors to pins 3 and 7 instead of wiring them direct to 5V - but it doesn't really matter.
// try to keep the wiring fairly short and neat.  If you must have longer wires, it helps to have a bypass capacitor 0.1uF connected close to the chip across the supply (pins 4 and 8).

#include <Arduino.h>

// you don't have to use pin D10 for SS in SPI master mode - but if you use a different pin, then you must still configure D10 as an output (and use it freely without affecting SPI)
// don't leave D10 as an input (or if you do make sure it is kept pulled high).
// You might have more than one SPI ram chip, - then you need extra SS outputs for each chip but you can still use D10 for one of them.
// this code then needs tweaking to enable the appropriate chip at read/write time
// if you do that, you probably won't have to bother about the set-up part as sequential mode is the SPI RAM chip default anyway - I only put the set-up part in for completeness. 

#define SPI_SS 10

class spiRam {
  public:
    static void start(void);
    static void writeSpi(uint8_t *src, uint32_t address, uint16_t count); // src is pointer to normal Arduino SRAM
    static void readSpi(uint8_t *dest, uint32_t address, uint16_t count); // dest is pointer to normal Arduino SRAM
  private:
    static void startTransmission(uint8_t *ptr, uint32_t address, uint16_t count, uint8_t opcode, uint8_t mode);    
  };
#endif
